import requests
import pandas as pd
from ipykernel.kernelbase import Kernel


__version__ = '0.1.0'

class ElasticsearchKernel(Kernel):
    implementation = 'elasticsearch_kernel'
    implementation_version = __version__
    language = 'sql'
    language_version = 'latest'
    language_info = {'name': 'sql',
                     'mimetype': 'text/x-sh',
                     'file_extension': '.sql'}
    banner = 'elasticsearch kernel'

    def __init__(self, **kwargs):
        Kernel.__init__(self, **kwargs)
        self.engine = False
        
    def output(self, output):
        if not self.silent:
            display_content = {'source': 'kernel',
                               'data': {'text/html': output},
                               'metadata': {}}
            self.send_response(self.iopub_socket, 'display_data', display_content)
    
    def ok(self):
        return {'status':'ok', 'execution_count':self.execution_count, 'payload':[], 'user_expressions':{}}

    def err(self, msg):
        return {'status':'error',
                'error':msg,
                'traceback':[msg],
                'execution_count':self.execution_count,
                'payload':[],
                'user_expressions':{}}

    def do_execute(self, code, silent, store_history=True, user_expressions=None, allow_stdin=False):
        self.silent = silent
        output = ''
        if not code.strip():
            return self.ok()
        sql = code.rstrip()+('' if code.rstrip().endswith(";") else ';')
        try:
            for v in sql.split(";"):
                v = v.rstrip()
                l = v.lower()
                if len(l)>0:
                    if l.startswith('elasticsearch://') or l.startswith('es://'):
                        self.engine = l.replace('elasticsearch', 'es').replace('es', 'http')
                        output = str(requests.get(self.engine).json())
                        self.engine += '/_sql'
                    else:
                        if self.engine:
                            t = requests.post(self.engine, json={"query":l}).json()
                            output = pd.DataFrame(t['rows'], columns=[i['name'] for i in t['columns']]).to_html()
                        else:
                            output = 'Unable to connect to Elasticsearch server. Check that the server is running.'
            self.output(output)
            return self.ok()
        except Exception as msg:
            self.output(str(msg))
            return self.err('Error executing code ' + sql)
